@@ -232,8 +232,10 @@ NAN_METHOD(WrappedRE2::New)
 	bool dotAll = false;
 	bool unicode = false;
 	bool sticky = false;
+	bool hasIndices = false;
 
 	auto context = Nan::GetCurrentContext();
+	bool needFlags = true;
 
 	if (info.Length() > 1)
 	{
@@ -272,9 +274,13 @@ NAN_METHOD(WrappedRE2::New)
 			case 'y':
 				sticky = true;
 				break;
+			case 'd':
+				hasIndices = true;
+				break;
 			}
 		}
 		size = 0;
+		needFlags = false;
 	}
 
 	bool needConversion = true;
@@ -299,13 +305,18 @@ NAN_METHOD(WrappedRE2::New)
 
 		source = escapeRegExp(data, size);
 
-		v8::RegExp::Flags flags = re->GetFlags();
-		global = bool(flags & v8::RegExp::kGlobal);
-		ignoreCase = bool(flags & v8::RegExp::kIgnoreCase);
-		multiline = bool(flags & v8::RegExp::kMultiline);
-		dotAll = bool(flags & v8::RegExp::kDotAll);
-		unicode = bool(flags & v8::RegExp::kUnicode);
-		sticky = bool(flags & v8::RegExp::kSticky);
+		if (needFlags)
+		{
+			v8::RegExp::Flags flags = re->GetFlags();
+			global = bool(flags & v8::RegExp::kGlobal);
+			ignoreCase = bool(flags & v8::RegExp::kIgnoreCase);
+			multiline = bool(flags & v8::RegExp::kMultiline);
+			dotAll = bool(flags & v8::RegExp::kDotAll);
+			unicode = bool(flags & v8::RegExp::kUnicode);
+			sticky = bool(flags & v8::RegExp::kSticky);
+			hasIndices = bool(flags & v8::RegExp::kHasIndices);
+			needFlags = false;
+		}
 	}
 	else if (info[0]->IsObject() && !info[0]->IsString())
 	{
@@ -326,12 +337,17 @@ NAN_METHOD(WrappedRE2::New)
 
 			source = re2->source;
 
-			global = re2->global;
-			ignoreCase = re2->ignoreCase;
-			multiline = re2->multiline;
-			dotAll = re2->dotAll;
-			unicode = true;
-			sticky = re2->sticky;
+			if (needFlags)
+			{
+				global = re2->global;
+				ignoreCase = re2->ignoreCase;
+				multiline = re2->multiline;
+				dotAll = re2->dotAll;
+				unicode = true;
+				sticky = re2->sticky;
+				hasIndices = re2->hasIndices;
+				needFlags = false;
+			}
 		}
 	}
 	else if (info[0]->IsString())
@@ -385,7 +401,7 @@ NAN_METHOD(WrappedRE2::New)
 	options.set_dot_nl(dotAll);
 	options.set_log_errors(false); // inappropriate when embedding
 
-	std::unique_ptr<WrappedRE2> re2(new WrappedRE2(re2::StringPiece(data, size), options, source, global, ignoreCase, multiline, dotAll, sticky));
+	std::unique_ptr<WrappedRE2> re2(new WrappedRE2(re2::StringPiece(data, size), options, source, global, ignoreCase, multiline, dotAll, sticky, hasIndices));
 	if (!re2->regexp.ok())
 	{
 		return Nan::ThrowSyntaxError(re2->regexp.error().c_str());
