@@ -1,6 +1,6 @@
 'use strict';
 
-const RE2 = require('./build/Release/re2');
+const RE2 = require('./build/Release/re2.node');
 
 if (typeof Symbol != 'undefined') {
   Symbol.match &&
@@ -19,7 +19,7 @@ if (typeof Symbol != 'undefined') {
     (RE2.prototype[Symbol.split] = function (str, limit) {
       return this.split(str, limit);
     });
-    Symbol.matchAll &&
+  Symbol.matchAll &&
     (RE2.prototype[Symbol.matchAll] = function* (str) {
       if (!this.global) {
         throw TypeError('String.prototype.matchAll called with a non-global RE2 argument');
@@ -29,6 +29,7 @@ if (typeof Symbol != 'undefined') {
       for (;;) {
         const result = re.exec(str);
         if (!result) break;
+        if (result[0] === '') ++re.lastIndex;
         yield result;
       }
     });
