@@ -13,7 +13,10 @@ unit.add(module, [
     'use strict';
 
     var re = new RE2(/apples/gi);
-    var result = re.replace('Apples are round, and apples are juicy.', 'oranges');
+    var result = re.replace(
+      'Apples are round, and apples are juicy.',
+      'oranges'
+    );
     eval(t.TEST("result === 'oranges are round, and oranges are juicy.'"));
 
     re = new RE2(/xmas/i);
@@ -199,15 +202,24 @@ unit.add(module, [
     'use strict';
 
     var re = new RE2(/яблоки/gi);
-    var result = re.replace(new Buffer('Яблоки красны, яблоки сочны.'), 'апельсины');
+    var result = re.replace(
+      new Buffer('Яблоки красны, яблоки сочны.'),
+      'апельсины'
+    );
     eval(t.TEST('result instanceof Buffer'));
     eval(t.TEST("result.toString() === 'апельсины красны, апельсины сочны.'"));
 
-    result = re.replace(new Buffer('Яблоки красны, яблоки сочны.'), new Buffer('апельсины'));
+    result = re.replace(
+      new Buffer('Яблоки красны, яблоки сочны.'),
+      new Buffer('апельсины')
+    );
     eval(t.TEST('result instanceof Buffer'));
     eval(t.TEST("result.toString() === 'апельсины красны, апельсины сочны.'"));
 
-    result = re.replace('Яблоки красны, яблоки сочны.', new Buffer('апельсины'));
+    result = re.replace(
+      'Яблоки красны, яблоки сочны.',
+      new Buffer('апельсины')
+    );
     eval(t.TEST("typeof result == 'string'"));
     eval(t.TEST("result === 'апельсины красны, апельсины сочны.'"));
   },
@@ -280,5 +292,89 @@ unit.add(module, [
 
     eval(t.TEST("re2.replace('ABCDEFABCDEF', '!') === '!!!!!FABCDEF'"));
     eval(t.TEST('re2.lastIndex === 0'));
+  },
+
+  // Non-matches
+
+  function test_replaceOneNonMatch(t) {
+    'use strict';
+
+    function replacer(match, capture, offset, string) {
+      t.test(typeof offset == 'number');
+      t.test(typeof match == 'string');
+      t.test(typeof string == 'string');
+      t.test(typeof capture == 'undefined');
+      t.test(offset === 0);
+      t.test(string === 'hello ');
+      return '';
+    }
+
+    var re = new RE2(/hello (world)?/);
+    re.replace('hello ', replacer);
+  },
+  function test_replaceTwoNonMatches(t) {
+    'use strict';
+
+    function replacer(match, capture1, capture2, offset, string, groups) {
+      t.test(typeof offset == 'number');
+      t.test(typeof match == 'string');
+      t.test(typeof string == 'string');
+      t.test(typeof capture1 == 'undefined');
+      t.test(typeof capture2 == 'undefined');
+      t.test(offset === 1);
+      t.test(match === 'b & y');
+      t.test(string === 'ab & yz');
+      t.test(typeof groups == 'object');
+      t.test(Object.keys(groups).length == 2);
+      t.test(groups.a === undefined);
+      t.test(groups.b == undefined);
+      return '';
+    }
+
+    var re = new RE2(/b(?<a>1)? & (?<b>2)?y/);
+    var result = re.replace('ab & yz', replacer);
+    eval(t.TEST("result === 'az'"));
+  },
+  function test_replaceGroupSimple(t) {
+    'use strict';
+
+    var re = new RE2(/(2)/);
+
+    var result = re.replace('123', '$0');
+    eval(t.TEST("result === '1$03'"));
+    result = re.replace('123', '$1');
+    eval(t.TEST("result === '123'"));
+    result = re.replace('123', '$2');
+    eval(t.TEST("result === '1$23'"));
+
+    result = re.replace('123', '$00');
+    eval(t.TEST("result === '1$003'"));
+    result = re.replace('123', '$01');
+    eval(t.TEST("result === '123'"));
+    result = re.replace('123', '$02');
+    eval(t.TEST("result === '1$023'"));
+  },
+  function test_replaceGroupCases(t) {
+    'use strict';
+
+    var re = new RE2(/(test)/g);
+    var result = re.replace('123', '$1$20');
+    eval(t.TEST("result === '123'"));
+
+    re = new RE2(/(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)/g);
+    result = re.replace('abcdefghijklmnopqrstuvwxyz123', '$10$20');
+    eval(t.TEST("result === 'jb0wo0123'"));
+
+    re = new RE2(/(.)(.)(.)(.)(.)/g);
+    result = re.replace('abcdefghijklmnopqrstuvwxyz123', '$10$20');
+    eval(t.TEST("result === 'a0b0f0g0k0l0p0q0u0v0z123'"));
+
+    re = new RE2(/(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)(.)/g);
+    result = re.replace('abcdefghijklmnopqrstuvwxyz123', '$10$20');
+    eval(t.TEST("result === 'jtvwxyz123'"));
+
+    re = new RE2(/abcd/g);
+    result = re.replace('abcd123', '$1$2');
+    eval(t.TEST("result === '$1$2123'"));
   }
 ]);
